/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.cloudfarming.client.isobus.tree;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPolygon;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.logging.Level;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.isobus.model.*;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.GeometryUtil;
import org.openide.util.NbBundle.Messages;

/**
 * Geographical representation of an Isobus partfield.
 * @author Gerben
 */
//FIXME Field class need to have an ID
//FIXME field class need to have  private void readObject(java.io.ObjectInputStream in)  throws IOException, ClassNotFoundException 
    
@Messages({
    "isobus partfieldgeographical taskdata node.name={0} - {1}",
    "isobus partfieldgeographical no polygon=Isobus partfield [{0}] does not contain a polygon to display as a geographical."
})
public class PartFieldGeographical implements Geometrical<Geometry> {

    private IsobusPartField isobusPartField;
    private final AppLogger LOG = AppLogFactory.getLogger(PartFieldGeographical.class);
    private Geometry geometry;
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    
    public PartFieldGeographical(IsobusPartField isobusPartField) {
        this.isobusPartField = isobusPartField;
    }

    @Override
    public com.vividsolutions.jts.geom.Point getCentroid() {
        Geometry temp = getGeometry();
        if(temp != null){
            return temp.getCentroid();
        }
        return null;
    }

    @Override
    public Geometry getRenderObject(Envelope envelope) {
        return geometry;
    }

    @Override
    public String getTooltipText() {
        return Bundle.isobus_partfieldgeographical_taskdata_node_name(isobusPartField.getCode(), isobusPartField.getDesignator());
    }

    // TODO: Isobus partfield can also have linestrings and points besides polygons
    //FIXME: this methods needs thorough testing!
    /**
     * Build wkt to create a vividsolutions Geometry.
     */
    @Override
    public Geometry getGeometry() {
        if (geometry == null) {

            // Polygon should have at least 1 coordinate.
            int counter = 0;
            
            if (isobusPartField != null) {
                StringBuilder wkt = new StringBuilder();
                wkt.append("MULTIPOLYGON(");

                for (IsobusPolygon polygon : isobusPartField.getPLN()) {
                    wkt.append("(");

                    for (IsobusLineString line : polygon.getLineString()) {
                        String firstPoint = "";
                        // Polygon exterior
                        if (line.getLineStringType() == LineStringTypeEnum.POLYGON_EXTERIOR) {
                            wkt.append("(");
                            for (IsobusPoint point : line.getPoints()) {
                                counter++;
                                StringBuilder pointStr = new StringBuilder();

                                pointStr.append(point.getPositionEast());
                                pointStr.append(" ");
                                pointStr.append(point.getPositionNorth());

                                if (firstPoint.equals("")) {
                                    firstPoint = pointStr.toString();
                                }

                                wkt.append(pointStr);
                                wkt.append(",");
                            }
                            
                            // Add first coordinate again to close the polygon
                            wkt.append(firstPoint);
                            wkt.append("),");
                        }

                    }
                    // Remove last char (,)
                    wkt.deleteCharAt(wkt.length() - 1);
                    wkt.append("),");
                }

                // Remove last char (,)
                wkt.deleteCharAt(wkt.length() - 1);
                wkt.append(")");

                // Polygon should have at least 1 coordinate.
                if(counter > 0){
//                    LOG.log(Level.INFO, "Extracted WKT: {0}", wkt.toString());
                    geometry = (MultiPolygon) GeometryUtil.wktToGeometry(wkt.toString());
                    return geometry;
                }
                else{
                    geometry = (MultiPolygon) GeometryUtil.wktToGeometry("MULTIPOLYGON EMPTY");
                    LOG.log(Level.INFO, Bundle.isobus_partfieldgeographical_no_polygon(isobusPartField.getDesignator()));
                }
            }
        }
        return geometry;
    }

    @Override
    public Geometry getBoundingBox() {
        return (this.geometry!=null)?geometry.getEnvelope():null;
    }

    @Override
    public String getIconPath() {
        return null;
    }

    @Override
    public String getIconLabel() {
        return null;
    }

    PropertyChangeSupport getPropertyChangeSupport(){
        if (propertyChangeSupport == null){
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    
    /**
     * Add PropertyChangeListener.
     *
     * @param listener
     */
    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    /**
     * Remove PropertyChangeListener.
     *
     * @param listener
     */
    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }
}
