/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.AllocationTime;
import nl.cloudfarming.client.isobus.model.DeviceElement;
import nl.cloudfarming.client.isobus.model.IsobusProduct;
import nl.cloudfarming.client.isobus.model.ProductAllocation;
import nl.cloudfarming.client.isobus.model.ValuePresentation;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;

/**
 * <p>Child factory for {@link ProductAllocation}</p>
 * <P>Possible children are {@link IsobusProduct}, {@link DeviceElement}, {@link ValuePresentation} and {@link AllocationTime}</p>
 * @author Gerben Feenstra
 */
public class ProductAllocationChildFactory extends ChildFactory<Node> {

    private final ProductAllocation productAllocation;

    public ProductAllocationChildFactory(ProductAllocation productAllocation) {
        this.productAllocation = productAllocation;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        //Product (PDT)
        IsobusProduct isobusProduct = (IsobusProduct) productAllocation.getProductIdRef();
        if(isobusProduct != null){
            toPopulate.add(new ProductNode(isobusProduct));
        }
        
        //DeviceElement (DET)
        DeviceElement deviceElement = (DeviceElement) productAllocation.getDeviceElementIdRef();
        if (deviceElement != null) {
            toPopulate.add(new DeviceElementNode(deviceElement));
        }

        //ValuePresentation (VPN)
        ValuePresentation presentation = (ValuePresentation) productAllocation.getValuePresentationIdRef();
        if (presentation != null) {
            toPopulate.add(new ValuePresentationNode(presentation));
        }

        //AllocationTime (ASP)
        AllocationTime allocationTime = productAllocation.getAllocationTimeStamp();
        if (allocationTime != null) {
            toPopulate.add(new AllocationStampNode(allocationTime));
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;

    }
}
